#ifndef VECTOR3HEADER
#define VECTOR3HEADER
#include "Matrix4x4.h"

class Matrix4x4;
#include <math.h>

class Vector3
{
	//our data
	float data[3];
	friend class Matrix4x4;

	friend Vector3 operator*(const float f, const Vector3 & v);

public:
	Vector3();
	~Vector3();
	Vector3(const float* f);
	Vector3(const float a, const float b, const float c);

	inline void Set(const float*f)
	{
		data[0] = f[0];
		data[1] = f[1];
		data[2] = f[2];
	}

	inline void Set(const float a, const float b, const float c)
	{
		data[0] = a;
		data[1] = b;
		data[2] = c;
	}

	inline void Set(const float f, const unsigned int i)
	{
		assert (i <= 2);
		data[i] = f;
	}

	inline void GetAll(float*f) const
	{
		f[0] = data[0];
		f[1] = data[1];
		f[2] = data[2];
	}

	inline float Get(const unsigned i) const
	{
		assert(i <= 2);
		return data[i];
	}

	void Interpolate(const Vector3 & v1, const Vector3 & v2, const float blend);

	inline const float * GetPtr() const {return data;}

	//get the magnitude
	inline float Magnitude() const
	{
		return sqrtf(data[0]*data[0]+data[1]*data[1]+data[2]*data[2]);
	}
	
	//normalize the vector
	void Normalize();
	
	//return v*this
	inline float Dot(const Vector3 & v) const
	{
		return (data[0] * v.data[0]+ data[1] * v.data[1]+ data[2] * v.data[2]);
	}
	
	//this = this x v
	void Cross(const Vector3 & v);

	//this = v1 x v2
	void Cross(const Vector3 & v1,const Vector3 & v2);

	//this = s*this
	inline void Mult(const float s)
	{
		data[0] *= s;
		data[1] *= s;
		data[2] *= s;
	}

	//this = m*this
	inline void Mult(const Matrix4x4 & m)
	{
		float a = m.data[0] * data[0] + m.data[1] * data[1] + m.data[2] * data[2] + m.data[3];
		float b = m.data[4] * data[0] + m.data[5] * data[1] + m.data[6] * data[2] + m.data[7];
		float c = m.data[8] * data[0] + m.data[9] * data[1] + m.data[10] * data[2] + m.data[11];
		data[0] = a;
		data[1] = b;
		data[2] = c;
	}
	
	//this = m*v
	inline void Mult(const Vector3 & v, const Matrix4x4 & m)
	{
		data[0] = m.data[0] * v.data[0] + m.data[1] * v.data[1] + m.data[2] * v.data[2] + m.data[3];
		data[1] = m.data[4] * v.data[0] + m.data[5] * v.data[1] + m.data[6] * v.data[2] + m.data[7];
		data[2] = m.data[8] * v.data[0] + m.data[9] * v.data[1] + m.data[10] * v.data[2] + m.data[11];
	}

	//this = m*v (less the last column of m)
	inline void MultRot(const Vector3 &v, const Matrix4x4 & m)
	{
		data[0] = m.data[0] * v.data[0] + m.data[1] * v.data[1] + m.data[2] * v.data[2];
		data[1] = m.data[4] * v.data[0] + m.data[5] * v.data[1] + m.data[6] * v.data[2];
		data[2] = m.data[8] * v.data[0] + m.data[9] * v.data[1] + m.data[10] * v.data[2];

	}

	//this += v
	inline void Add(const Vector3 & v)
	{
		data[0] += v.data[0];
		data[1] += v.data[1];
		data[2] += v.data[2];
	}
	
	//this = v1+v2
	void Add(const Vector3 & v1,const Vector3 & v2);
	
	inline void Add(const float x,const float y,const float z)
	{
		data[0] += x;
		data[1] += y;
		data[2] += z;
	}
	//this -= v
	void Sub(const Vector3 & v);

	//this = v1-v2
	void Sub(const Vector3 & v1,const Vector3 & v2);

	//returns the cartesian distance from this to v
	float Distance(const Vector3 & v) const;

	void Print(FILE * f) const;


};



#endif